import bt

# 模拟从前端接收到的因子列表
factors = [
    {'name': 'SMA', 'period': 50},
    {'name': 'EMA', 'period': 200},
    {'name': 'RSI', 'period': 14},
    {'name': 'BollingerBands', 'period': 20}
]


def create_indicator(name, **kwargs):
    """
    根据名称和参数动态创建bt的技术指标。
    """
    if name == 'SMA':
        return bt.indicators.SMA(kwargs['period'])
    elif name == 'EMA':
        return bt.indicators.EMA(kwargs['period'])
    elif name == 'RSI':
        return bt.indicators.RSI(kwargs['period'])
    elif name == 'BollingerBands':
        return bt.indicators.BollingerBands(kwargs['period'])

    else:
        raise ValueError(f"未知的指标名称: {name}")


def create_dynamic_strategy(name, factors):
    """
    根据传递的因子列表动态构建策略。
    """
    algos = [bt.algos.RunMonthly(),  # 每月运行一次
             bt.algos.SelectAll()]  # 选择所有资产

    # 动态生成指标选择逻辑
    for factor in factors:
        indicator = create_indicator(factor['name'], **factor)
        if factor['name'] == 'RSI':
            # RSI 特定的买入/卖出逻辑
            buy_signal = bt.algos.SelectWhere(indicator < 30)  # 超卖区域买入
            sell_signal = bt.algos.SelectWhere(indicator > 70)  # 超买区域卖出
            algos.append(buy_signal)
            algos.append(bt.algos.WeighTarget(1.0))
            algos.append(bt.algos.Rebalance())
            algos.append(sell_signal)
            algos.append(bt.algos.WeighTarget(0.0))
            algos.append(bt.algos.Rebalance())
        elif factor['name'] == 'BollingerBands':
            # 布林带特定的逻辑（买入接近下轨，卖出接近上轨）
            buy_signal = bt.algos.SelectWhere(bt.data < indicator['lower'])
            sell_signal = bt.algos.SelectWhere(bt.data > indicator['upper'])
            algos.append(buy_signal)
            algos.append(bt.algos.WeighTarget(1.0))
            algos.append(bt.algos.Rebalance())
            algos.append(sell_signal)
            algos.append(bt.algos.WeighTarget(0.0))
            algos.append(bt.algos.Rebalance())
        # 可以为其他指标添加更多逻辑

    return bt.Strategy(name, algos)


# 创建动态策略
dynamic_strategy = create_dynamic_strategy("Dynamic_Strategy", factors)

# 获取数据
data = bt.get('spy,agg', start='2020-01-01', end='2023-01-01')

# 创建回测
backtest = bt.Backtest(dynamic_strategy, data)

# 运行回测
result = bt.run(backtest)

# 展示结果
result.display()
result.plot()

