import asyncio
import json
from datetime import datetime

import bt
import numpy as np
import pandas as pd

from src.utils.backtest_until import get_local_data, convert_pandas_to_json_serializable
from src.models import wance_data_storage_backtest, wance_data_stock
from src.tortoises_orm_config import init_tortoise


# 布林带策略函数
async def create_bollinger_bands_strategy(data, stock_code: str, bollingerMA: int = 50, std_dev: int = 200):
    # 生成布林带策略信号
    signal = await bollinger_bands_strategy(data, bollingerMA, std_dev)

    # 使用bt框架构建策略
    strategy = bt.Strategy(f'{stock_code} 布林带策略',
                           [bt.algos.RunDaily(),
                            bt.algos.SelectAll(),  # 选择所有股票
                            bt.algos.WeighTarget(signal),  # 根据信号调整权重
                            bt.algos.Rebalance()])  # 调仓
    return strategy, signal


async def bollinger_bands_strategy(df, window=20, num_std_dev=2):
    """
    基于布林带策略生成买卖信号。

    参数:
    df: pd.DataFrame, 股票的价格数据，行索引为日期，列为股票代码。
    window: int, 计算布林带中轨线的窗口期。
    num_std_dev: float, 标准差的倍数，用于计算上下轨。

    返回:
    signal: pd.DataFrame, 每只股票的买卖信号，1 表示买入，0 表示卖出。
    """
    # 计算中轨线（移动平均）
    middle_band = df.rolling(window=window, min_periods=1).mean()

    # 计算滚动标准差
    rolling_std = df.rolling(window=window, min_periods=1).std()

    # 计算上轨线和下轨线
    upper_band = middle_band + (rolling_std * num_std_dev)
    lower_band = middle_band - (rolling_std * num_std_dev)

    # 初始化信号 DataFrame
    signal = pd.DataFrame(index=df.index, columns=df.columns)

    # 生成买入信号：当价格突破下轨时
    for column in df.columns:
        signal[column] = np.where(df[column] < lower_band[column], 1, np.nan)  # 买入信号

    # 生成卖出信号：当价格突破上轨时
    for column in df.columns:
        signal[column] = np.where(df[column] > upper_band[column], 0, signal[column])  # 卖出信号

    # 前向填充信号，持仓不变
    signal = signal.ffill()

    # 将剩余的 NaN 替换为 0
    signal = signal.fillna(0)

    return signal


async def storage_backtest_data(source_column_name, result, signal, stock_code, stock_data_series, bollingerMA,
                                std_dev):
    await init_tortoise()

    # 要存储的字段列表
    fields_to_store = [
        'stock_code', 'strategy_name', 'stock_close_price', 'daily_price',
        'price', 'returns', 'data_start_time', 'data_end_time',
        'backtest_end_time', 'position', 'backtest_name', 'rf', 'total_return', 'cagr',
        'max_drawdown', 'calmar', 'mtd', 'three_month',
        'six_month', 'ytd', 'one_year', 'three_year',
        'five_year', 'ten_year', 'incep', 'daily_sharpe',
        'daily_sortino', 'daily_mean', 'daily_vol',
        'daily_skew', 'daily_kurt', 'best_day', 'worst_day',
        'monthly_sharpe', 'monthly_sortino', 'monthly_mean',
        'monthly_vol', 'monthly_skew', 'monthly_kurt',
        'best_month', 'worst_month', 'yearly_sharpe',
        'yearly_sortino', 'yearly_mean', 'yearly_vol',
        'yearly_skew', 'yearly_kurt', 'best_year', 'worst_year',
        'avg_drawdown', 'avg_drawdown_days', 'avg_up_month',
        'avg_down_month', 'win_year_perc', 'twelve_month_win_perc'
    ]

    # 准备要存储的数据
    data_to_store = {
        'stock_code': stock_code,
        'strategy_name': "布林带策略",
        'stock_close_price': json.dumps(stock_data_series.fillna(0).rename_axis('time').reset_index().assign(
            time=stock_data_series.index.strftime('%Y%m%d')).set_index('time').to_dict(orient='index')),
        'daily_price': convert_pandas_to_json_serializable(result[source_column_name].daily_prices),
        'price': convert_pandas_to_json_serializable(result[source_column_name].prices),
        'returns': convert_pandas_to_json_serializable(result[source_column_name].returns.fillna(0)),
        'data_start_time': pd.to_datetime(result.stats.loc["start"].iloc[0]).strftime('%Y%m%d'),
        'data_end_time': pd.to_datetime(result.stats.loc["end"].iloc[0]).strftime('%Y%m%d'),
        'backtest_end_time': int(datetime.now().strftime('%Y%m%d')),
        'position': convert_pandas_to_json_serializable(signal),
        'backtest_name': f'{stock_code} 布林带策略 MA{bollingerMA}-{std_dev}倍标准差',
        'indicator_type': 'Bollinger',
        'indicator_information': json.dumps({'bollingerMA': bollingerMA, 'std_dev': std_dev})
    }

    # 使用循环填充其他字段
    for field in fields_to_store[12:]:  # 从第10个字段开始
        value = result.stats.loc[field].iloc[0]
        data_to_store[field] = 0.0 if (isinstance(value, float) and np.isnan(value)) else value

    # 检查是否存在该 backtest_name
    existing_record = await wance_data_storage_backtest.WanceDataStorageBacktest.filter(
        backtest_name=data_to_store['backtest_name']
    ).first()

    if existing_record:
        # 如果存在，更新记录
        await wance_data_storage_backtest.WanceDataStorageBacktest.filter(
            id=existing_record.id
        ).update(**data_to_store)
    else:
        # 如果不存在，创建新的记录
        await wance_data_storage_backtest.WanceDataStorageBacktest.create(**data_to_store)

    return data_to_store


async def run_bollinger_backtest(field_list: list,
                                 stock_list: list,
                                 period: str = '1d',
                                 start_time: str = '',
                                 end_time: str = '',
                                 count: int = 100,
                                 dividend_type: str = 'none',
                                 fill_data: bool = True,
                                 data_dir: str = '',
                                 bollingerMA: int = 50,
                                 std_dev: int = 200):
    try:
        # 初始化一个列表用于存储每只股票的回测结果字典
        results_list = []

        # 遍历每只股票的数据（每列代表一个股票的收盘价）
        data = await get_local_data(field_list, stock_list, period, start_time, end_time, count, dividend_type,
                                    fill_data,
                                    data_dir)

        for stock_code in stock_list:

            data_column_name = f'close_{stock_code}'
            source_column_name = f'{stock_code} 布林带策略'
            backtest_name = f'{stock_code} 布林带策略 MA{bollingerMA}-{std_dev}倍标准差'
            now_time = int(datetime.now().strftime('%Y%m%d'))
            db_result = await wance_data_storage_backtest.WanceDataStorageBacktest.filter(
                backtest_name=backtest_name)
            if db_result:
                if db_result[0].backtest_end_time == now_time:
                    results_list.append({source_column_name: db_result[0]})

            # elif data_column_name in data.columns:
            if data_column_name in data.columns:
                stock_data_series = data[[data_column_name]]  # 提取该股票的收盘价 DataFrame
                stock_data_series.columns = ['close']  # 重命名列为 'close'

                # 创建布林带策略
                strategy, signal = await create_bollinger_bands_strategy(stock_data_series, stock_code,
                                                                         bollingerMA=bollingerMA,
                                                                         std_dev=std_dev)
                # 创建回测
                backtest = bt.Backtest(strategy=strategy, data=stock_data_series, initial_capital=100000)
                # 运行回测
                result = bt.run(backtest)
                # 存储回测结果
                data_to_store = await storage_backtest_data(source_column_name, result, signal, stock_code,
                                                            stock_data_series,
                                                            bollingerMA, std_dev)
                # # 绘制回测结果图表
                # result.plot()
                # # 绘制个别股票数据图表
                # plt.figure(figsize=(12, 6))
                # plt.plot(stock_data_series.index, stock_data_series['close'], label='Stock Price')
                # plt.title(f'Stock Price for {stock_code}')
                # plt.xlabel('Date')
                # plt.ylabel('Price')
                # plt.legend()
                # plt.grid(True)
                # plt.show()
                # 将结果存储为字典并添加到列表中
                results_list.append({source_column_name: data_to_store})

            else:
                print(f"数据中缺少列: {data_column_name}")

        return results_list  # 返回结果列表

    except Exception as e:
        print(f"Error occurred: {e}")


async def start_bollinger_backtest_service(field_list: list,
                                 stock_list: list,
                                 period: str = '1d',
                                 start_time: str = '',
                                 end_time: str = '',
                                 count: int = -1,
                                 dividend_type: str = 'none',
                                 fill_data: bool = True,
                                 data_dir: str = '',
                                 bollingerMA: int = 50,
                                 std_dev: int = 200):
    for stock_code in stock_list:
        backtest_name = f'{stock_code} 布林带策略 MA{bollingerMA}-{std_dev}倍标准差'
        db_result = await wance_data_storage_backtest.WanceDataStorageBacktest.filter(
            backtest_name=backtest_name)
        now_time = int(datetime.now().strftime('%Y%m%d'))

        if db_result and db_result[0].backtest_end_time == now_time:
            return db_result
        else:
            # 执行回测
            result = await run_bollinger_backtest(
                field_list=field_list,
                stock_list=stock_list,
                period=period,
                start_time=start_time,
                end_time=end_time,
                count=count,
                dividend_type=dividend_type,
                fill_data=fill_data,
                data_dir=data_dir,
                bollingerMA=bollingerMA,
                std_dev=std_dev,
            )
            return result


async def init_backtest_db():
    bollinger_list = [{"bollingerMA": 20, "std_dev": 2}, {"bollingerMA": 30, "std_dev": 2},
                      {"bollingerMA": 70, "std_dev": 2}, {"bollingerMA": 5, "std_dev": 1},
                      {"bollingerMA": 20, "std_dev": 3}, {"bollingerMA": 50, "std_dev": 2.5}]
    await init_tortoise()
    wance_db = await wance_data_stock.WanceDataStock.all()
    bollinger_list_lenght = len(bollinger_list)

    for stock_code in wance_db:
        for i in range(bollinger_list_lenght):
            bollingerMA = bollinger_list[i]['bollingerMA']
            std_dev = bollinger_list[i]['std_dev']
            source_column_name = f'{stock_code} 布林带策略 MA{bollingerMA}-{std_dev}倍标准差'
            result = await run_bollinger_backtest(field_list=['close', 'time'],
                                                  stock_list=[stock_code.stock_code],
                                                  bollingerMA=bollingerMA,
                                                  std_dev=std_dev)

            print(f"回测成功 {source_column_name}")


if __name__ == '__main__':
    # 测试类的回测
    asyncio.run(run_bollinger_backtest(field_list=['close', 'time'],
                                       stock_list=['601222.SH', '601677.SH'],
                                       bollingerMA=20,
                                       std_dev=2))

    # # 初始化数据库表
    # asyncio.run(init_backtest_db())
