import asyncio
import json
from datetime import datetime

import bt  # 引入bt框架用于回测
import numpy as np
import pandas as pd

from src.models import wance_data_storage_backtest, wance_data_stock, user_combination_history  # 导入数据库模型
from src.tortoises_orm_config import init_tortoise  # 初始化数据库
from src.utils.combination_until import get_local_data  # 获取本地数据的工具函数


# 创建布林带策略函数
async def create_bollinger_bands_strategy(data, stock_weights_dict, bollingerMA: int = 50, std_dev: int = 200):
    """
    创建组合布林带策略，并根据股票权重进行回测。

    参数:
    data: pd.DataFrame, 股票的价格数据，行索引为日期，列为股票代码。
    stock_weights_dict: dict, 每只股票的权重字典。
    bollingerMA: int, 用于计算布林带的移动平均线周期。
    std_dev: int, 用于计算布林带上下轨的标准差倍数。

    返回:
    strategy: bt.Strategy, 构建的策略。
    combined_signal: pd.DataFrame, 股票的买卖信号。
    """
    # 创建一个数据框用于存储所有股票的买卖信号
    combined_signal = pd.DataFrame(index=data.index)

    # 遍历每只股票，根据股票权重生成布林带信号
    for stock_code in stock_weights_dict.keys():
        if f'{stock_code}' in data.columns:
            # 提取每只股票的数据
            stock_data_series = data[[f'{stock_code}']]
            stock_data_series.columns = [f'{stock_code}']

            # 调用布林带策略计算买卖信号
            signal = await bollinger_bands_strategy(stock_data_series, bollingerMA, std_dev)
            combined_signal[stock_code] = signal
        else:
            print(f"Warning: Stock code {stock_code} not found in data columns.")  # 如果股票不在数据中，发出警告

    # 创建权重数据框
    weights = pd.DataFrame(index=data.index, data=[stock_weights_dict] * len(data.index))

    # 使用bt框架构建布林带策略
    strategy = bt.Strategy('组合布林带策略',
                           [bt.algos.RunDaily(),  # 每天运行策略
                            bt.algos.SelectAll(),  # 选择所有可用的股票
                            bt.algos.WeighSpecified(**stock_weights_dict),  # 根据给定权重进行加权
                            bt.algos.Rebalance()])  # 调整投资组合

    return strategy, combined_signal  # 返回构建的策略和信号


async def bollinger_bands_strategy(df, window=20, num_std_dev=2):
    """
    基于布林带策略生成买卖信号。

    参数:
    df: pd.DataFrame, 股票的价格数据，行索引为日期，列为股票代码。
    window: int, 计算布林带中轨线的窗口期。
    num_std_dev: float, 标准差的倍数，用于计算上下轨。

    返回:
    signal: pd.DataFrame, 每只股票的买卖信号，1 表示买入，0 表示卖出。
    """
    # 计算布林带的中轨（移动平均）
    middle_band = df.rolling(window=window, min_periods=1).mean()
    # 计算移动标准差
    rolling_std = df.rolling(window=window, min_periods=1).std()
    # 计算布林带的上下轨
    upper_band = middle_band + (rolling_std * num_std_dev)
    lower_band = middle_band - (rolling_std * num_std_dev)

    # 创建一个数据框用于存储每只股票的买卖信号
    signal = pd.DataFrame(index=df.index, columns=df.columns)

    # 根据布林带计算买卖信号
    for column in df.columns:
        signal[column] = np.where(df[column] < lower_band[column], 1, np.nan)  # 当收盘价低于下轨，生成买入信号
        signal[column] = np.where(df[column] > upper_band[column], 0, signal[column])  # 当收盘价高于上轨，生成卖出信号

    # 前向填充信号，并用0填充NaN值
    signal = signal.ffill().fillna(0)
    return signal  # 返回信号数据框


async def storage_backtest_data(user_id, stock_weights, strategy_parame, strategy_name):
    """
    存储回测结果到数据库。

    参数:
    user_id: int, 用户的ID。
    stock_weights: dict, 股票权重字典。
    strategy_parame: dict, 策略参数字典。
    strategy_name: str, 策略名称。
    """
    await init_tortoise()  # 初始化数据库

    # 查询用户的已有记录
    existing_record = await user_combination_history.UserCombinationHistory.filter(user_id=user_id).first()

    # 将股票权重和策略参数转换为JSON格式
    stock_weights_json = json.dumps(stock_weights)
    strategy_parame_json = json.dumps(strategy_parame)

    # 如果记录存在，更新记录；否则，创建新记录
    if existing_record:
        await user_combination_history.UserCombinationHistory.filter(
            id=existing_record.id
        ).update(user_id=user_id,
                 last_time=int(datetime.now().strftime('%Y%m%d')),
                 stock_weights=stock_weights_json,
                 strategy_name=strategy_name,
                 strategy_parame=strategy_parame_json)
    else:
        await user_combination_history.UserCombinationHistory.create(user_id=user_id,
                                                                     last_time=int(datetime.now().strftime('%Y%m%d')),
                                                                     stock_weights=stock_weights_json,
                                                                     strategy_name=strategy_name,
                                                                     strategy_parame=strategy_parame_json)


async def run_combined_bollinger_backtest(field_list: list,
                                          stock_list: list,
                                          stock_weights: list,
                                          user_id: int = 1,
                                          period: str = '1d',
                                          start_time: str = '',
                                          end_time: str = '',
                                          count: int = 100,
                                          dividend_type: str = 'none',
                                          fill_data: bool = True,
                                          data_dir: str = '',
                                          bollingerMA: int = 50,
                                          std_dev: int = 200):
    """
    运行组合布林带策略的回测。

    参数:
    field_list: list, 需要获取的字段列表。
    stock_list: list, 需要回测的股票代码列表。
    stock_weights: list, 股票权重列表。
    user_id: int, 用户ID。
    period: str, 数据周期。
    start_time: str, 开始时间。
    end_time: str, 结束时间。
    count: int, 数据数量限制。
    dividend_type: str, 分红类型。
    fill_data: bool, 是否填充数据。
    data_dir: str, 数据目录。
    bollingerMA: int, 布林带移动平均线周期。
    std_dev: int, 标准差倍数。

    返回:
    result: 回测结果。
    """
    try:
        # 获取本地数据
        data = await get_local_data(field_list, stock_list, period, start_time, end_time, count, dividend_type,
                                    fill_data, data_dir)

        # 获取股票权重字典
        stock_weights_dict = stock_weights[0] if stock_weights else {}

        # 创建布林带策略
        strategy, signal = await create_bollinger_bands_strategy(data, stock_weights_dict, bollingerMA, std_dev)

        # 创建回测对象
        backtest = bt.Backtest(strategy, data, initial_capital=100000)

        # 策略参数
        strategy_parame = {"bollingerMA": bollingerMA, "std_dev": std_dev}
        strategy_name = "布林带策略"

        # 运行回测
        result = bt.run(backtest)

        # 存储回测结果
        await storage_backtest_data(user_id, stock_weights_dict, strategy_parame, strategy_name)

        return result  # 返回回测结果

    except Exception as e:
        print(f"Error occurred: {e}")  # 输出错误信息


async def start_bollinger_combination_service(field_list: list,
                                              stock_list: list,
                                              stock_weights: list,
                                              user_id: int = 1,
                                              period: str = '1d',
                                              start_time: str = '',
                                              end_time: str = '',
                                              count: int = -1,
                                              dividend_type: str = 'none',
                                              fill_data: bool = True,
                                              data_dir: str = '',
                                              bollingerMA: int = 50,
                                              std_dev: int = 200):
    """
    启动布林带策略回测服务。

    参数:
    field_list: list, 需要获取的字段列表。
    stock_list: list, 需要回测的股票代码列表。
    stock_weights: list, 股票权重列表。
    user_id: int, 用户ID。
    period: str, 数据周期。
    start_time: str, 开始时间。
    end_time: str, 结束时间。
    count: int, 数据数量限制。
    dividend_type: str, 分红类型。
    fill_data: bool, 是否填充数据。
    data_dir: str, 数据目录。
    bollingerMA: int, 布林带移动平均线周期。
    std_dev: int, 标准差倍数。

    返回:
    result: 回测结果。
    """
    # 调用回测函数并返回结果
    result = await run_combined_bollinger_backtest(
        field_list=field_list,
        stock_list=stock_list,
        stock_weights=stock_weights,
        user_id=user_id,
        period=period,
        start_time=start_time,
        end_time=end_time,
        count=count,
        dividend_type=dividend_type,
        fill_data=fill_data,
        data_dir=data_dir,
        bollingerMA=bollingerMA,
        std_dev=std_dev,
    )
    return result  # 返回回测结果


if __name__ == '__main__':
    # 主函数，启动回测服务
    asyncio.run(run_combined_bollinger_backtest(field_list=['close', 'time'],
                                                stock_list=["688031.SH", "600025.SH", "601222.SH"],
                                                user_id=1,
                                                stock_weights=[{"688031.SH": 0.5, "600025.SH": 0.3, "601222.SH": 0.2}],
                                                bollingerMA=50,
                                                std_dev=200))
