import asyncio
import json
from datetime import datetime

import bt
import numpy as np
import pandas as pd

from src.models import wance_data_storage_backtest, wance_data_stock, user_combination_history
from src.tortoises_orm_config import init_tortoise
from src.utils.backtest_until import convert_pandas_to_json_serializable
from src.utils.combination_until import get_local_data


# 组合双均线策略函数
async def create_combined_sma_strategy(data, stock_weights_dict, short_window: int = 50, long_window: int = 200):
    """
    创建组合双均线策略，并根据股票权重进行回测。

    参数:
    data: pd.DataFrame, 股票价格数据，行索引为日期，列为股票代码。
    stock_weights_dict: dict, 股票权重字典，键为股票代码，值为权重。
    short_window: int, 短期均线窗口期，默认为50。
    long_window: int, 长期均线窗口期，默认为200。

    返回:
    strategy: bt.Strategy, 创建的双均线策略对象。
    combined_signal: pd.DataFrame, 每只股票的买卖信号数据框。
    """
    combined_signal = pd.DataFrame(index=data.index)  # 初始化信号数据框

    # 遍历每只股票代码，根据股票权重生成信号
    for stock_code in stock_weights_dict.keys():
        # 确保每个股票代码唯一，不要使用重复代码
        if f'{stock_code}' in data.columns:
            stock_data_series = data[[f'{stock_code}']]  # 提取对应股票的数据
            stock_data_series.columns = [f'{stock_code}']  # 重命名列

            signal = await dual_ma_strategy(stock_data_series, short_window, long_window)  # 生成信号
            combined_signal[stock_code] = signal  # 存储信号
        else:
            print(f"Warning: Stock code {stock_code} not found in data columns.")  # 提示未找到的股票代码

    # 将权重转换为 DataFrame，行索引为数据的时间，列为股票代码
    weights = pd.DataFrame(index=data.index, data=[stock_weights_dict] * len(data.index))

    # 使用bt框架构建组合策略
    strategy = bt.Strategy('组合双均线策略',
                           [bt.algos.RunDaily(),  # 每天运行
                            bt.algos.SelectAll(),  # 选择所有股票
                            bt.algos.WeighSpecified(**stock_weights_dict),  # 根据权重调整股票权重
                            bt.algos.Rebalance()])  # 调仓

    return strategy, combined_signal  # 返回策略和信号


async def dual_ma_strategy(df, short_window=20, long_window=50):
    """
    基于双均线策略生成买卖信号。

    参数:
    df: pd.DataFrame, 股票的价格数据，行索引为日期，列为股票代码。
    short_window: int, 短期均线窗口期。
    long_window: int, 长期均线窗口期。

    返回:
    signal: pd.DataFrame, 每只股票的买卖信号，1 表示买入，0 表示卖出。
    """
    # 计算短期和长期均线
    short_ma = df.rolling(window=short_window, min_periods=1).mean()  # 短期均线
    long_ma = df.rolling(window=long_window, min_periods=1).mean()  # 长期均线

    # 生成买入和卖出信号
    buy_signal = np.where(short_ma > long_ma, 1, np.nan)  # 买入信号
    sell_signal = np.where(short_ma < long_ma, 0, np.nan)  # 卖出信号

    # 将信号转换为DataFrame，并向前填充
    signal = pd.DataFrame(buy_signal, index=df.index, columns=df.columns)
    signal = np.where(short_ma < long_ma, 0, signal)  # 卖出信号更新
    signal = pd.DataFrame(signal, index=df.index, columns=df.columns).ffill()  # 前向填充信号

    signal = signal.fillna(0)  # 将 NaN 替换为 0
    return signal  # 返回信号


async def storage_backtest_data(user_id, result, stock_data_series, source_column_name, signal, stock_weights,
                                strategy_parame, strategy_name,short_window,long_window):
    """
    将回测数据存储到数据库。

    参数:
    user_id: int, 用户ID。
    stock_weights: dict, 股票权重字典。
    strategy_parame: dict, 策略参数字典。
    strategy_name: str, 策略名称。
    """
    await init_tortoise()  # 初始化数据库连接

    # 要存储的字段列表
    fields_to_store = [
        'stock_weights', 'strategy_name', 'stock_close_price', 'daily_price',
        'price', 'returns', 'data_start_time', 'data_end_time',
        'backtest_end_time', 'position', 'backtest_name', 'rf', 'total_return', 'cagr',
        'max_drawdown', 'calmar', 'mtd', 'three_month',
        'six_month', 'ytd', 'one_year', 'three_year',
        'five_year', 'ten_year', 'incep', 'daily_sharpe',
        'daily_sortino', 'daily_mean', 'daily_vol',
        'daily_skew', 'daily_kurt', 'best_day', 'worst_day',
        'monthly_sharpe', 'monthly_sortino', 'monthly_mean',
        'monthly_vol', 'monthly_skew', 'monthly_kurt',
        'best_month', 'worst_month', 'yearly_sharpe',
        'yearly_sortino', 'yearly_mean', 'yearly_vol',
        'yearly_skew', 'yearly_kurt', 'best_year', 'worst_year',
        'avg_drawdown', 'avg_drawdown_days', 'avg_up_month',
        'avg_down_month', 'win_year_perc', 'twelve_month_win_perc'
    ]

    # 准备要存储的数据
    data_to_store = {
        'stock_weights': stock_weights,
        'strategy_name': "双均线策略",
        'stock_close_price': json.dumps(stock_data_series.fillna(0).rename_axis('time').reset_index().assign(
            time=stock_data_series.index.strftime('%Y%m%d')).set_index('time').to_dict(orient='index')),
        'daily_price': convert_pandas_to_json_serializable(result[source_column_name].daily_prices),
        'price': convert_pandas_to_json_serializable(result[source_column_name].prices),
        'returns': convert_pandas_to_json_serializable(result[source_column_name].returns.fillna(0)),
        'data_start_time': pd.to_datetime(result.stats.loc["start"].iloc[0]).strftime('%Y%m%d'),
        'data_end_time': pd.to_datetime(result.stats.loc["end"].iloc[0]).strftime('%Y%m%d'),
        'backtest_end_time': int(datetime.now().strftime('%Y%m%d')),
        'position': convert_pandas_to_json_serializable(signal),
        'backtest_name': f'{stock_weights} 双均线策略 MA{short_window}-{long_window}日',
        'indicator_type': 'SMA',
        'indicator_information': json.dumps({'short_window': short_window, 'long_window': long_window})
    }

    # 使用循环填充其他字段
    for field in fields_to_store[12:]:  # 从第10个字段开始
        value = result.stats.loc[field].iloc[0]
        data_to_store[field] = 0.0 if (isinstance(value, float) and np.isnan(value)) else value

    # 检查是否存在该 backtest_name
    existing_record = await user_combination_history.UserCombinationHistory.filter(user_id=user_id).first()

    stock_weights_json = json.dumps(stock_weights)  # 将权重字典转换为JSON格式
    strategy_parame_json = json.dumps(strategy_parame)  # 将策略参数字典转换为JSON格式

    if existing_record:
        # 如果存在，更新记录
        await user_combination_history.UserCombinationHistory.filter(
            id=existing_record.id
        ).update(user_id=user_id,
                 last_time=int(datetime.now().strftime('%Y%m%d')),  # 更新时间
                 stock_weights=stock_weights_json,
                 strategy_name=strategy_name,
                 strategy_parame=strategy_parame_json)
    else:
        # 如果不存在，创建新的记录
        await user_combination_history.UserCombinationHistory.create(user_id=user_id,
                                                                     last_time=int(datetime.now().strftime('%Y%m%d')),
                                                                     stock_weights=stock_weights_json,
                                                                     strategy_name=strategy_name,
                                                                     strategy_parame=strategy_parame_json)
    return data_to_store


async def run_combined_sma_backtest(field_list: list,
                                    stock_list: list,
                                    stock_weights: list,
                                    user_id: int = 1,
                                    period: str = '1d',
                                    start_time: str = '',
                                    end_time: str = '',
                                    count: int = 100,
                                    dividend_type: str = 'none',
                                    fill_data: bool = True,
                                    data_dir: str = '',
                                    short_window: int = 50,
                                    long_window: int = 200):
    """
    运行组合双均线策略的回测。

    参数:
    field_list: list, 需要获取的字段列表。
    stock_list: list, 股票代码列表。
    stock_weights: list, 股票权重列表。
    user_id: int, 用户ID，默认为1。
    period: str, 数据时间周期，默认为'1d'。
    start_time: str, 开始时间，默认为''（空字符串表示不限制）。
    end_time: str, 结束时间，默认为''（空字符串表示不限制）。
    count: int, 获取的数据数量，默认为100。
    dividend_type: str, 分红类型，默认为'none'。
    fill_data: bool, 是否填充缺失数据，默认为True。
    data_dir: str, 数据目录，默认为''（空字符串表示当前目录）。
    short_window: int, 短期均线窗口期，默认为50。
    long_window: int, 长期均线窗口期，默认为200。

    返回:
    result: bt.BacktestResult, 回测结果。
    """
    try:
        # 获取本地数据
        data = await get_local_data(field_list, stock_list, period, start_time, end_time, count, dividend_type,
                                    fill_data, data_dir)

        # 将 stock_weights 转换为字典形式，提取第一个字典作为权重
        stock_weights_dict = stock_weights[0] if stock_weights else {}

        # 创建组合双均线策略
        strategy, signal = await create_combined_sma_strategy(data, stock_weights_dict, short_window, long_window)

        # 创建回测
        backtest = bt.Backtest(strategy, data, initial_capital=100000)  # 初始化回测对象

        strategy_parame = {"short_window": short_window, "long_window": long_window}  # 策略参数

        strategy_name = "双均线策略"  # 策略名称

        # 运行回测
        result = bt.run(backtest)  # 执行回测

        # 存储回测数据
        result = await storage_backtest_data(user_id=user_id, result=result, stock_data_series=data,
                                             source_column_name="组合双均线策略",
                                             signal=signal, stock_weights=stock_weights_dict,
                                             strategy_parame=strategy_parame,
                                             strategy_name=strategy_name,short_window=short_window,long_window=long_window)

        return result  # 返回回测结果

    except Exception as e:
        print(f"Error occurred: {e}")  # 打印错误信息


async def start_sma_combination_service(field_list: list,
                                        stock_list: list,
                                        stock_weights: list,
                                        user_id: int = 1,
                                        period: str = '1d',
                                        start_time: str = '',
                                        end_time: str = '',
                                        count: int = -1,
                                        dividend_type: str = 'none',
                                        fill_data: bool = True,
                                        data_dir: str = '',
                                        short_window: int = 50,
                                        long_window: int = 200):
    """
    启动双均线组合策略服务。

    参数:
    field_list: list, 需要获取的字段列表。
    stock_list: list, 股票代码列表。
    stock_weights: list, 股票权重列表。
    user_id: int, 用户ID，默认为1。
    period: str, 数据时间周期，默认为'1d'。
    start_time: str, 开始时间，默认为''（空字符串表示不限制）。
    end_time: str, 结束时间，默认为''（空字符串表示不限制）。
    count: int, 获取的数据数量，默认为-1（表示不限制）。
    dividend_type: str, 分红类型，默认为'none'。
    fill_data: bool, 是否填充缺失数据，默认为True。
    data_dir: str, 数据目录，默认为''（空字符串表示当前目录）。
    short_window: int, 短期均线窗口期，默认为50。
    long_window: int, 长期均线窗口期，默认为200。

    返回:
    result: bt.BacktestResult, 回测结果。
    """
    result = await run_combined_sma_backtest(
        field_list=field_list,
        stock_list=stock_list,
        user_id=user_id,
        stock_weights=stock_weights,
        period=period,
        start_time=start_time,
        end_time=end_time,
        count=count,
        dividend_type=dividend_type,
        fill_data=fill_data,
        data_dir=data_dir,
        short_window=short_window,
        long_window=long_window,
    )
    return result  # 返回回测结果


async def init_backtest_db():
    """
    初始化回测数据库，针对每个股票和不同的均线组合运行回测。

    """
    # 定义短期和长期均线组合
    sma_list = [{"short_window": 5, "long_window": 10}, {"short_window": 10, "long_window": 30},
                {"short_window": 30, "long_window": 60}, {"short_window": 30, "long_window": 90},
                {"short_window": 70, "long_window": 140}, {"short_window": 120, "long_window": 250}]

    await init_tortoise()  # 初始化数据库连接
    wance_db = await wance_data_stock.WanceDataStock.all()  # 获取所有股票数据
    sma_list_length = len(sma_list)  # 获取均线组合的数量

    # 遍历每只股票，针对每个均线组合运行回测
    for stock_code in wance_db:
        for i in range(sma_list_length):
            short_window = sma_list[i]['short_window']  # 短期均线
            long_window = sma_list[i]['long_window']  # 长期均线
            source_column_name = f'{stock_code} 双均线策略 MA{short_window}-{long_window}日'  # 结果列名称

            # 运行回测
            result = await run_combined_sma_backtest(field_list=['close', 'time'],
                                                     stock_list=[stock_code.stock_code],
                                                     short_window=short_window,
                                                     long_window=long_window)

            print(f"回测成功 {source_column_name}")  # 打印回测成功信息


if __name__ == '__main__':
    # 测试类的回测
    # asyncio.run(run_sma_backtest(field_list=['close', 'time'],
    #                              stock_list=['601222.SH', '601677.SH'],
    #                              short_window=10,
    #                              long_window=30))

    # 初始化数据库表
    # asyncio.run(init_backtest_db())
    asyncio.run(run_combined_sma_backtest(field_list=['close', 'time'],
                                          stock_list=["688031.SH", "600025.SH", "601222.SH"],
                                          user_id=1,
                                          stock_weights=[{"688031.SH": 0.5, "600025.SH": 0.3, "601222.SH": 0.2}],
                                          short_window=10,
                                          long_window=30))
