import asyncio
import json
from datetime import datetime

import bt
import numpy as np
import pandas as pd

from src.models import wance_data_storage_backtest, wance_data_stock, user_combination_history
from src.tortoises_orm_config import init_tortoise
from src.utils.combination_until import get_local_data


# 创建组合MACD策略函数
async def create_combined_macd_strategy(data, stock_weights_dict, short_window: int = 12, long_window: int = 26, signal_window: int = 9):
    """
    创建组合MACD策略，并根据股票权重进行回测。

    参数:
    data: pd.DataFrame, 股票价格数据，行索引为日期，列为股票代码。
    stock_weights_dict: dict, 股票权重字典，键为股票代码，值为相应权重。
    short_window: int, 短期EMA窗口期，默认为12。
    long_window: int, 长期EMA窗口期，默认为26。
    signal_window: int, 信号线窗口期，默认为9。

    返回:
    strategy: bt.Strategy, 创建的组合MACD策略。
    combined_signal: pd.DataFrame, 各股票的买卖信号。
    """
    combined_signal = pd.DataFrame(index=data.index)  # 创建一个空的DataFrame用于存放买卖信号

    # 遍历每个股票代码，计算对应的MACD信号
    for stock_code in stock_weights_dict.keys():
        if stock_code in data.columns:
            stock_data_series = data[[stock_code]]  # 获取该股票的价格数据
            stock_data_series.columns = [stock_code]  # 重命名列

            signal = await macd_strategy(stock_data_series, short_window, long_window, signal_window)  # 计算MACD信号
            combined_signal[stock_code] = signal  # 将信号存入combined_signal
        else:
            print(f"Warning: Stock code {stock_code} not found in data columns.")  # 如果股票代码不存在，发出警告

    # 将权重转换为 DataFrame，行索引为数据的时间，列为股票代码
    weights = pd.DataFrame(index=data.index, data=[stock_weights_dict] * len(data.index))

    # 使用bt框架构建组合策略
    strategy = bt.Strategy('组合MACD策略',
                           [bt.algos.RunDaily(),  # 每日运行策略
                            bt.algos.SelectAll(),  # 选择所有股票
                            bt.algos.WeighSpecified(**stock_weights_dict),  # 按照指定的权重分配
                            bt.algos.Rebalance()])  # 调整持仓

    return strategy, combined_signal  # 返回创建的策略和买卖信号


# MACD策略生成买卖信号
async def macd_strategy(df, short_window=12, long_window=26, signal_window=9):
    """
    基于MACD策略生成买卖信号。

    参数:
    df: pd.DataFrame, 股票的价格数据，行索引为日期，列为股票代码。
    short_window: int, 短期EMA窗口期，默认为12。
    long_window: int, 长期EMA窗口期，默认为26。
    signal_window: int, 信号线窗口期，默认为9。

    返回:
    signal: pd.DataFrame, 每只股票的买卖信号，1 表示买入，-1 表示卖出。
    """
    # 计算短期和长期EMA
    short_ema = df.ewm(span=short_window, adjust=False).mean()  # 短期指数移动平均
    long_ema = df.ewm(span=long_window, adjust=False).mean()  # 长期指数移动平均

    # 计算MACD和信号线
    macd = short_ema - long_ema  # MACD值
    signal_line = macd.ewm(span=signal_window, adjust=False).mean()  # 信号线

    # 生成买卖信号
    signal = np.where(macd > signal_line, 1, 0)  # MACD上穿信号线 -> 买入
    signal = np.where(macd < signal_line, -1, signal)  # MACD下穿信号线 -> 卖出
    signal = pd.DataFrame(signal, index=df.index, columns=df.columns).ffill()  # 前向填充信号

    signal = signal.fillna(0)  # 将所有NaN替换为0
    return signal  # 返回买卖信号


# 保存回测数据
async def storage_backtest_data(user_id, stock_weights, strategy_parame, strategy_name):
    """
    将回测结果保存到数据库。

    参数:
    user_id: int, 用户ID。
    stock_weights: list, 股票权重。
    strategy_parame: dict, 策略参数。
    strategy_name: str, 策略名称。
    """
    await init_tortoise()  # 初始化Tortoise ORM

    # 检查是否存在该 backtest_name
    existing_record = await user_combination_history.UserCombinationHistory.filter(user_id=user_id).first()

    # 将股票权重和策略参数转换为JSON格式
    stock_weights_json = json.dumps(stock_weights)
    strategy_parame_json = json.dumps(strategy_parame)

    if existing_record:  # 如果存在记录，则更新
        await user_combination_history.UserCombinationHistory.filter(
            id=existing_record.id
        ).update(user_id=user_id,
                 last_time=int(datetime.now().strftime('%Y%m%d')),
                 stock_weights=stock_weights_json,
                 strategy_name=strategy_name,
                 strategy_parame=strategy_parame_json)
    else:  # 如果不存在记录，则创建新的记录
        await user_combination_history.UserCombinationHistory.create(user_id=user_id,
                                                                     last_time=int(datetime.now().strftime('%Y%m%d')),
                                                                     stock_weights=stock_weights_json,
                                                                     strategy_name=strategy_name,
                                                                     strategy_parame=strategy_parame_json)


# 运行组合MACD策略回测
async def run_combined_macd_backtest(field_list: list,
                                     stock_list: list,
                                     stock_weights: list,
                                     user_id: int = 1,
                                     period: str = '1d',
                                     start_time: str = '',
                                     end_time: str = '',
                                     count: int = 100,
                                     dividend_type: str = 'none',
                                     fill_data: bool = True,
                                     data_dir: str = '',
                                     short_window: int = 12,
                                     long_window: int = 26,
                                     signal_window: int = 9):
    """
    运行组合MACD策略的回测。

    参数:
    field_list: list, 需要获取的字段列表。
    stock_list: list, 股票代码列表。
    stock_weights: list, 股票权重列表。
    user_id: int, 用户ID，默认为1。
    period: str, 数据时间周期，默认为'1d'。
    start_time: str, 开始时间，默认为''（空字符串表示不限制）。
    end_time: str, 结束时间，默认为''（空字符串表示不限制）。
    count: int, 获取的数据数量，默认为100。
    dividend_type: str, 分红类型，默认为'none'。
    fill_data: bool, 是否填充缺失数据，默认为True。
    data_dir: str, 数据目录，默认为''（空字符串表示当前目录）。
    short_window: int, 短期EMA窗口期，默认为12。
    long_window: int, 长期EMA窗口期，默认为26。
    signal_window: int, 信号线窗口期，默认为9。

    返回:
    result: bt.BacktestResult, 回测结果。
    """
    try:
        # 获取本地数据
        data = await get_local_data(field_list, stock_list, period, start_time, end_time, count, dividend_type,
                                     fill_data, data_dir)

        # 将 stock_weights 转换为字典形式，提取第一个字典作为权重
        stock_weights_dict = stock_weights[0] if stock_weights else {}

        # 创建组合MACD策略
        strategy, signal = await create_combined_macd_strategy(data, stock_weights_dict, short_window, long_window, signal_window)

        # 创建回测
        backtest = bt.Backtest(strategy, data, initial_capital=100000)  # 使用初始资本创建回测

        # 策略参数和名称
        strategy_parame = {"short_window": short_window, "long_window": long_window, "signal_window": signal_window}
        strategy_name = "MACD策略"

        # 运行回测
        result = bt.run(backtest)  # 运行回测

        # 保存回测数据
        await storage_backtest_data(user_id, stock_weights_dict, strategy_parame, strategy_name)

        return result  # 返回回测结果

    except Exception as e:
        print(f"Error occurred: {e}")  # 捕获并打印异常


# 启动MACD策略服务
async def start_macd_combination_service(field_list: list,
                                         stock_list: list,
                                         stock_weights: list,
                                         user_id: int = 1,
                                         period: str = '1d',
                                         start_time: str = '',
                                         end_time: str = '',
                                         count: int = -1,
                                         dividend_type: str = 'none',
                                         fill_data: bool = True,
                                         data_dir: str = '',
                                         short_window: int = 12,
                                         long_window: int = 26,
                                         signal_window: int = 9):
    """
    启动MACD策略组合服务。

    参数:
    field_list: list, 需要获取的字段列表。
    stock_list: list, 股票代码列表。
    stock_weights: list, 股票权重列表。
    user_id: int, 用户ID，默认为1。
    period: str, 数据时间周期，默认为'1d'。
    start_time: str, 开始时间，默认为''（空字符串表示不限制）。
    end_time: str, 结束时间，默认为''（空字符串表示不限制）。
    count: int, 获取的数据数量，默认为-1（表示获取所有可用数据）。
    dividend_type: str, 分红类型，默认为'none'。
    fill_data: bool, 是否填充缺失数据，默认为True。
    data_dir: str, 数据目录，默认为''（空字符串表示当前目录）。
    short_window: int, 短期EMA窗口期，默认为12。
    long_window: int, 长期EMA窗口期，默认为26。
    signal_window: int, 信号线窗口期，默认为9。

    返回:
    result: bt.BacktestResult, 回测结果。
    """
    result = await run_combined_macd_backtest(
        field_list=field_list,
        stock_list=stock_list,
        user_id=user_id,
        stock_weights=stock_weights,
        period=period,
        start_time=start_time,
        end_time=end_time,
        count=count,
        dividend_type=dividend_type,
        fill_data=fill_data,
        data_dir=data_dir,
        short_window=short_window,
        long_window=long_window,
        signal_window=signal_window
    )
    return result  # 返回回测结果


if __name__ == '__main__':
    # 运行回测
    asyncio.run(run_combined_macd_backtest(field_list=['close', 'time'],
                                            stock_list=["688031.SH", "600025.SH", "601222.SH"],
                                            user_id=1,
                                            stock_weights=[{"688031.SH": 0.5, "600025.SH": 0.3, "601222.SH": 0.2}],
                                            short_window=12,
                                            long_window=26,
                                            signal_window=9))
