import asyncio
import json
from datetime import datetime

import bt
import numpy as np
import pandas as pd

from src.models import wance_data_storage_backtest, wance_data_stock, user_combination_history
from src.tortoises_orm_config import init_tortoise
from src.utils.combination_until import get_local_data

# 组合反双均线策略函数
async def create_combined_sma_strategy(data, stock_weights_dict, short_window: int = 50, long_window: int = 200):
    """
    创建组合反双均线策略，并根据股票权重进行回测。

    参数:
    data: pd.DataFrame, 包含股票的历史价格数据，行索引为时间，列为股票代码。
    stock_weights_dict: dict, 包含每只股票的权重。
    short_window: int, 短期均线的计算窗口，默认为50。
    long_window: int, 长期均线的计算窗口，默认为200。

    返回:
    strategy: bt.Strategy, 创建的组合策略。
    combined_signal: pd.DataFrame, 每只股票的买卖信号。
    """
    # 创建一个 DataFrame 用于存储组合信号，索引为数据的日期
    combined_signal = pd.DataFrame(index=data.index)

    # 遍历每只股票，根据其权重生成信号
    for stock_code in stock_weights_dict.keys():
        # 确保每个股票代码在数据中唯一
        if f'{stock_code}' in data.columns:
            # 选择对应股票的数据列
            stock_data_series = data[[f'{stock_code}']]
            stock_data_series.columns = [f'{stock_code}']

            # 调用反双均线策略生成买卖信号
            signal = await reverse_dual_ma_strategy(stock_data_series, short_window, long_window)
            # 将生成的信号添加到组合信号 DataFrame 中
            combined_signal[stock_code] = signal
        else:
            print(f"Warning: Stock code {stock_code} not found in data columns.")

    # 将权重字典转换为 DataFrame，行索引为数据的时间，列为股票代码
    weights = pd.DataFrame(index=data.index, data=[stock_weights_dict] * len(data.index))

    # 使用 bt 框架构建组合策略
    strategy = bt.Strategy('组合反双均线策略',
                           [bt.algos.RunDaily(),  # 每天运行策略
                            bt.algos.SelectAll(),  # 选择所有股票
                            bt.algos.WeighSpecified(**stock_weights_dict),  # 使用指定的权重
                            bt.algos.Rebalance()])  # 调仓

    return strategy, combined_signal


# 反双均线策略生成买卖信号
async def reverse_dual_ma_strategy(df, short_window=20, long_window=50):
    """
    基于反双均线策略生成买卖信号。

    参数:
    df: pd.DataFrame, 股票的价格数据，行索引为日期，列为股票代码。
    short_window: int, 短期均线窗口期。
    long_window: int, 长期均线窗口期。

    返回:
    signal: pd.DataFrame, 每只股票的买卖信号，1 表示买入，0 表示卖出。
    """
    # 计算短期和长期均线
    short_ma = df.rolling(window=short_window, min_periods=1).mean()
    long_ma = df.rolling(window=long_window, min_periods=1).mean()

    # 反双均线策略的买卖信号生成规则
    buy_signal = np.where(short_ma < long_ma, 1, np.nan)  # 短期均线跌破长期均线 -> 买入信号
    sell_signal = np.where(short_ma > long_ma, 0, np.nan)  # 短期均线上穿长期均线 -> 卖出信号

    # 创建信号 DataFrame
    signal = pd.DataFrame(buy_signal, index=df.index, columns=df.columns)
    # 将卖出信号应用到信号 DataFrame 中
    signal = np.where(short_ma > long_ma, 0, signal)
    signal = pd.DataFrame(signal, index=df.index, columns=df.columns).ffill()  # 向前填充信号

    signal = signal.fillna(0)  # 将 NaN 转换为 0
    return signal


# 保存回测数据
async def storage_backtest_data(user_id, stock_weights, strategy_parame, strategy_name):
    """
    保存回测数据到数据库。

    参数:
    user_id: int, 用户的 ID。
    stock_weights: dict, 股票权重。
    strategy_parame: dict, 策略参数。
    strategy_name: str, 策略名称。
    """
    await init_tortoise()  # 初始化 Tortoise ORM

    # 检查是否存在该 backtest_name
    existing_record = await user_combination_history.UserCombinationHistory.filter(user_id=user_id).first()

    # 将权重和策略参数转换为 JSON 格式
    stock_weights_json = json.dumps(stock_weights)
    strategy_parame_json = json.dumps(strategy_parame)

    if existing_record:
        # 如果记录已存在，更新记录
        await user_combination_history.UserCombinationHistory.filter(
            id=existing_record.id
        ).update(user_id=user_id,
                 last_time=int(datetime.now().strftime('%Y%m%d')),
                 stock_weights=stock_weights_json,
                 strategy_name=strategy_name,
                 strategy_parame=strategy_parame_json)
    else:
        # 如果记录不存在，创建新的记录
        await user_combination_history.UserCombinationHistory.create(user_id=user_id,
                                                                     last_time=int(datetime.now().strftime('%Y%m%d')),
                                                                     stock_weights=stock_weights_json,
                                                                     strategy_name=strategy_name,
                                                                     strategy_parame=strategy_parame_json)


# 运行组合反双均线策略回测
async def run_combined_sma_backtest(field_list: list,
                                    stock_list: list,
                                    stock_weights: list,
                                    user_id: int = 1,
                                    period: str = '1d',
                                    start_time: str = '',
                                    end_time: str = '',
                                    count: int = 100,
                                    dividend_type: str = 'none',
                                    fill_data: bool = True,
                                    data_dir: str = '',
                                    short_window: int = 50,
                                    long_window: int = 200):
    """
    运行组合反双均线策略的回测。

    参数:
    field_list: list, 包含需要获取的字段，如收盘价、时间等。
    stock_list: list, 需要回测的股票列表。
    stock_weights: list, 包含每只股票的权重字典。
    user_id: int, 用户 ID，默认为 1。
    period: str, 数据时间周期，默认为 '1d'。
    start_time: str, 开始时间，默认为 ''（空字符串表示不限制）。
    end_time: str, 结束时间，默认为 ''（空字符串表示不限制）。
    count: int, 获取的数据数量，默认为 100。
    dividend_type: str, 分红类型，默认为 'none'。
    fill_data: bool, 是否填充缺失数据，默认为 True。
    data_dir: str, 数据目录，默认为 ''（空字符串表示当前目录）。
    short_window: int, 短期均线窗口，默认为 50。
    long_window: int, 长期均线窗口，默认为 200。

    返回:
    result: bt.BacktestResult, 回测结果。
    """
    try:
        # 获取本地数据
        data = await get_local_data(field_list, stock_list, period, start_time, end_time, count, dividend_type,
                                    fill_data, data_dir)

        # 将 stock_weights 转换为字典形式，提取第一个字典作为权重
        stock_weights_dict = stock_weights[0] if stock_weights else {}

        # 创建组合反双均线策略
        strategy, signal = await create_combined_sma_strategy(data, stock_weights_dict, short_window, long_window)

        # 创建回测实例
        backtest = bt.Backtest(strategy, data, initial_capital=100000)

        # 策略参数
        strategy_parame = {"short_window": short_window, "long_window": long_window}

        strategy_name = "反双均线策略"

        # 运行回测
        result = bt.run(backtest)

        # 保存回测数据
        await storage_backtest_data(user_id, stock_weights_dict, strategy_parame, strategy_name)

        return result

    except Exception as e:
        print(f"Error occurred: {e}")


# 启动反双均线策略服务
async def start_reverse_SMA_combination_service(field_list: list,
                                                stock_list: list,
                                                stock_weights: list,
                                                user_id: int = 1,
                                                period: str = '1d',
                                                start_time: str = '',
                                                end_time: str = '',
                                                count: int = -1,
                                                dividend_type: str = 'none',
                                                fill_data: bool = True,
                                                data_dir: str = '',
                                                short_window: int = 50,
                                                long_window: int = 200):
    """
    启动反双均线策略服务，运行回测并返回结果。

    参数:
    field_list: list, 包含需要获取的字段。
    stock_list: list, 需要回测的股票列表。
    stock_weights: list, 股票权重列表。
    user_id: int, 用户 ID，默认为 1。
    period: str, 数据时间周期，默认为 '1d'。
    start_time: str, 开始时间，默认为 ''（空字符串表示不限制）。
    end_time: str, 结束时间，默认为 ''（空字符串表示不限制）。
    count: int, 获取的数据数量，默认为 -1（表示全部）。
    dividend_type: str, 分红类型，默认为 'none'。
    fill_data: bool, 是否填充缺失数据，默认为 True。
    data_dir: str, 数据目录，默认为 ''（空字符串表示当前目录）。
    short_window: int, 短期均线窗口，默认为 50。
    long_window: int, 长期均线窗口，默认为 200。

    返回:
    result: bt.BacktestResult, 回测结果。
    """
    result = await run_combined_sma_backtest(
        field_list=field_list,
        stock_list=stock_list,
        user_id=user_id,
        stock_weights=stock_weights,
        period=period,
        start_time=start_time,
        end_time=end_time,
        count=count,
        dividend_type=dividend_type,
        fill_data=fill_data,
        data_dir=data_dir,
        short_window=short_window,
        long_window=long_window,
    )
    return result


if __name__ == '__main__':
    # 运行回测
    asyncio.run(run_combined_sma_backtest(field_list=['close', 'time'],
                                          stock_list=["688031.SH", "600025.SH", "601222.SH"],
                                          user_id=1,
                                          stock_weights=[{"688031.SH": 0.5, "600025.SH": 0.3, "601222.SH": 0.2}],
                                          short_window=10,
                                          long_window=30))
