import asyncio
import json
from datetime import datetime

import bt  # 引入bt框架以进行回测
import numpy as np
import pandas as pd

from src.models import wance_data_storage_backtest, wance_data_stock, user_combination_history  # 导入数据库模型
from src.tortoises_orm_config import init_tortoise  # 初始化数据库
from src.utils.combination_until import get_local_data  # 获取本地数据的工具函数


# 创建RSI策略函数
async def create_rsi_strategy(data, stock_weights_dict, rsi_period: int = 14, overbought: float = 70,
                              oversold: float = 30):
    """
    创建组合RSI策略，并根据股票权重进行回测。

    参数:
    data: pd.DataFrame, 股票的价格数据，行索引为日期，列为股票代码。
    stock_weights_dict: dict, 每只股票的权重字典。
    rsi_period: int, 用于计算RSI的周期，默认值为14。
    overbought: float, 超买阈值，默认值为70。
    oversold: float, 超卖阈值，默认值为30。

    返回:
    strategy: bt.Strategy, 构建的RSI策略。
    combined_signal: pd.DataFrame, 股票的买卖信号。
    """
    # 创建一个数据框用于存储所有股票的买卖信号
    combined_signal = pd.DataFrame(index=data.index)

    # 遍历每只股票，生成RSI信号
    for stock_code in stock_weights_dict.keys():
        if f'{stock_code}' in data.columns:
            # 提取每只股票的数据
            stock_data_series = data[[f'{stock_code}']]
            stock_data_series.columns = [f'{stock_code}']

            # 调用RSI策略计算买卖信号
            signal = await rsi_strategy(stock_data_series, rsi_period, overbought, oversold)
            combined_signal[stock_code] = signal  # 存储信号
        else:
            print(f"Warning: Stock code {stock_code} not found in data columns.")  # 股票未在数据中，发出警告

    # 创建权重数据框
    weights = pd.DataFrame(index=data.index, data=[stock_weights_dict] * len(data.index))

    # 使用bt框架构建RSI策略
    strategy = bt.Strategy('组合RSI策略',
                           [bt.algos.RunDaily(),  # 每天运行策略
                            bt.algos.SelectAll(),  # 选择所有可用的股票
                            bt.algos.WeighSpecified(**stock_weights_dict),  # 根据给定权重进行加权
                            bt.algos.Rebalance()])  # 调整投资组合

    return strategy, combined_signal  # 返回构建的策略和信号


async def rsi_strategy(df, period=14, overbought=70, oversold=30):
    """
    基于RSI策略生成买卖信号。

    参数:
    df: pd.DataFrame, 股票的价格数据，行索引为日期，列为股票代码。
    period: int, RSI计算的周期，默认值为14。
    overbought: float, 超买阈值，默认值为70。
    oversold: float, 超卖阈值，默认值为30。

    返回:
    signal: pd.DataFrame, 每只股票的买卖信号，1 表示买入，0 表示卖出。
    """
    # 计算价格变化量
    delta = df.diff()
    # 计算上涨和下跌的平均值
    gain = (delta.where(delta > 0, 0)).rolling(window=period).mean()  # 上涨的平均值
    loss = (-delta.where(delta < 0, 0)).rolling(window=period).mean()  # 下跌的平均值

    # 计算相对强弱指数RS和RSI
    rs = gain / loss
    rsi = 100 - (100 / (1 + rs))  # 计算RSI

    # 创建一个数据框用于存储每只股票的买卖信号
    signal = pd.DataFrame(index=df.index, columns=df.columns)

    # 根据RSI计算买卖信号
    for column in df.columns:
        signal[column] = np.where(rsi[column] < oversold, 1, np.nan)  # 当RSI低于超卖阈值，生成买入信号
        signal[column] = np.where(rsi[column] > overbought, 0, signal[column])  # 当RSI高于超买阈值，生成卖出信号

    # 前向填充信号，并用0填充NaN值
    signal = signal.ffill().fillna(0)
    return signal  # 返回信号数据框


async def storage_backtest_data(user_id, stock_weights, strategy_parame, strategy_name):
    """
    存储回测结果到数据库。

    参数:
    user_id: int, 用户的ID。
    stock_weights: dict, 股票权重字典。
    strategy_parame: dict, 策略参数字典。
    strategy_name: str, 策略名称。
    """
    await init_tortoise()  # 初始化数据库

    # 查询用户的已有记录
    existing_record = await user_combination_history.UserCombinationHistory.filter(user_id=user_id).first()

    # 将股票权重和策略参数转换为JSON格式
    stock_weights_json = json.dumps(stock_weights)
    strategy_parame_json = json.dumps(strategy_parame)

    # 如果记录存在，更新记录；否则，创建新记录
    if existing_record:
        await user_combination_history.UserCombinationHistory.filter(
            id=existing_record.id
        ).update(user_id=user_id,
                 last_time=int(datetime.now().strftime('%Y%m%d')),
                 stock_weights=stock_weights_json,
                 strategy_name=strategy_name,
                 strategy_parame=strategy_parame_json)
    else:
        await user_combination_history.UserCombinationHistory.create(user_id=user_id,
                                                                     last_time=int(datetime.now().strftime('%Y%m%d')),
                                                                     stock_weights=stock_weights_json,
                                                                     strategy_name=strategy_name,
                                                                     strategy_parame=strategy_parame_json)


async def run_combined_rsi_backtest(field_list: list,
                                    stock_list: list,
                                    stock_weights: list,
                                    user_id: int = 1,
                                    period: str = '1d',
                                    start_time: str = '',
                                    end_time: str = '',
                                    count: int = 100,
                                    dividend_type: str = 'none',
                                    fill_data: bool = True,
                                    data_dir: str = '',
                                    rsi_period: int = 14,
                                    overbought: float = 70,
                                    oversold: float = 30):
    """
    运行组合RSI策略的回测。

    参数:
    field_list: list, 需要获取的字段列表。
    stock_list: list, 需要回测的股票代码列表。
    stock_weights: list, 股票权重列表。
    user_id: int, 用户ID，默认为1。
    period: str, 数据周期，默认为'1d'。
    start_time: str, 开始时间，默认为''。
    end_time: str, 结束时间，默认为''。
    count: int, 数据数量限制，默认为100。
    dividend_type: str, 分红类型，默认为'none'。
    fill_data: bool, 是否填充数据，默认为True。
    data_dir: str, 数据目录，默认为''。
    rsi_period: int, RSI计算周期，默认为14。
    overbought: float, 超买阈值，默认为70。
    oversold: float, 超卖阈值，默认为30。

    返回:
    result: 回测结果。
    """
    try:
        # 获取本地数据
        data = await get_local_data(field_list, stock_list, period, start_time, end_time, count, dividend_type,
                                    fill_data, data_dir)

        # 获取股票权重字典
        stock_weights_dict = stock_weights[0] if stock_weights else {}

        # 创建RSI策略
        strategy, signal = await create_rsi_strategy(data, stock_weights_dict, rsi_period, overbought, oversold)

        # 创建回测对象
        backtest = bt.Backtest(strategy, data, initial_capital=100000)

        # 策略参数
        strategy_parame = {"rsi_period": rsi_period, "overbought": overbought, "oversold": oversold}
        strategy_name = "RSI策略"

        # 运行回测
        result = bt.run(backtest)

        # 存储回测结果
        await storage_backtest_data(user_id, stock_weights_dict, strategy_parame, strategy_name)

        return result  # 返回回测结果

    except Exception as e:
        print(f"Error occurred: {e}")  # 输出错误信息


async def start_rsi_combination_service(field_list: list,
                                        stock_list: list,
                                        stock_weights: list,
                                        user_id: int = 1,
                                        period: str = '1d',
                                        start_time: str = '',
                                        end_time: str = '',
                                        count: int = -1,
                                        dividend_type: str = 'none',
                                        fill_data: bool = True,
                                        data_dir: str = '',
                                        rsi_period: int = 14,
                                        overbought: float = 70,
                                        oversold: float = 30):
    """
    启动RSI组合策略服务。

    参数:
    field_list: list, 需要获取的字段列表。
    stock_list: list, 需要回测的股票代码列表。
    stock_weights: list, 股票权重列表。
    user_id: int, 用户ID，默认为1。
    period: str, 数据周期，默认为'1d'。
    start_time: str, 开始时间，默认为''。
    end_time: str, 结束时间，默认为''。
    count: int, 数据数量限制，默认为-1。
    dividend_type: str, 分红类型，默认为'none'。
    fill_data: bool, 是否填充数据，默认为True。
    data_dir: str, 数据目录，默认为''。
    rsi_period: int, RSI计算周期，默认为14。
    overbought: float, 超买阈值，默认为70。
    oversold: float, 超卖阈值，默认为30。

    返回:
    result: 回测结果。
    """
    # 调用回测函数并返回结果
    result = await run_combined_rsi_backtest(
        field_list=field_list,
        stock_list=stock_list,
        stock_weights=stock_weights,
        user_id=user_id,
        period=period,
        start_time=start_time,
        end_time=end_time,
        count=count,
        dividend_type=dividend_type,
        fill_data=fill_data,
        data_dir=data_dir,
        rsi_period=rsi_period,
        overbought=overbought,
        oversold=oversold,
    )
    return result  # 返回回测结果


if __name__ == '__main__':
    # 主函数，启动回测服务
    asyncio.run(run_combined_rsi_backtest(field_list=['close', 'time'],
                                          stock_list=["688031.SH", "600025.SH", "601222.SH"],
                                          user_id=1,
                                          stock_weights=[{"688031.SH": 0.5, "600025.SH": 0.3, "601222.SH": 0.2}],
                                          rsi_period=14,
                                          overbought=70,
                                          oversold=30))
