from datetime import datetime
from decimal import Decimal
from enum import Enum, IntEnum
from typing import Optional

from pydantic import BaseModel

from src.settings.config import settings


def with_table_name(table_name: str):
    return f"{settings.DATABASE_PREFIX}{table_name}"


class Decimals:
    MAX_DIGITS = 15
    DECIMAL_PLACES = 3
    DECIMAL_PLACES_MORE = 6


class SecurityPriceCode(Enum):
    TRANSFER_FEE = "fee_1"  # 过户费

    STAMP_TAX = "fee_2"  # 印花税

    TOTAL_COMMISSION = "csf_fee"  # 总证券佣金


class UserSecurityExtraReminded(BaseModel):
    """
        reminded_target_buy_price: 最近提醒目标买入价
        reminded_target_buy_time: 目标买入价最近提醒时间
        reminded_buy_price: 最近提醒买入价
        reminded_buy_fluctuation: 最近提醒买入波动
        reminded_buy_time: 买入最近提醒时间
        reminded_sell_price: 最近提醒目标卖出价
        reminded_sell_fluctuation: 最近提醒卖出波动
        reminded_sell_time: 卖出最近提醒时间
        buy_up_first_interval: 买入上涨初次提醒波动间隔
        buy_up_continue_interval: 买入继续上涨提醒波动间隔
        buy_down_first_interval: 买入下跌初次提醒波动间隔
        buy_down_continue_interval: 买入继续下跌提醒波动间隔
        sell_down_first_interval: 卖出下跌初次提醒波动间隔
        sell_down_continue_interval: 卖出继续下跌提醒波动间隔
        browse_msg_alarm_flag: 浏览器消息提醒通知设置
        wx_msg_alarm_flag: 微信公众号消息提醒通知设置
    """
    reminded_target_buy_price: int = 0
    reminded_target_buy_time: Optional[datetime] = None
    reminded_buy_price: Decimal = 0.000
    reminded_buy_fluctuation: int = 0
    reminded_buy_time: Optional[datetime] = None
    reminded_sell_price: Decimal = 0.000
    reminded_sell_fluctuation: int = 0
    reminded_sell_time: Optional[datetime] = None
    buy_up_first_interval: int = 1
    buy_up_continue_interval: int = 1
    buy_down_first_interval: int = -1
    buy_down_continue_interval: int = -1
    sell_down_first_interval: int = -1
    sell_down_continue_interval: int = -1
    browse_msg_alarm_flag: bool = True
    wx_msg_alarm_flag: bool = True

    class Config:
        json_encoders = {Decimal: str}


class Status(IntEnum):
    """
    状态
    0-正常
    1-删除
    2-禁用
    3-退市
    """
    # 正常
    NORMAL = 0
    STOP = 1
    DISABLED = 2
    DELISTED = 3


class StockType(str, Enum):
    """
    股票类型
    """
    SH = "sh"
    SZ = "sz"


class UserSecurityAccountType(IntEnum):
    """
    用户中国证券账户账户类型
    """

    NORMAL = 0  # 普通账户

    MARGIN = 1  # 融资账户


class ExchangeType(str, Enum):
    """
    交易类型
    """
    SH = "SH"
    SZ = "SZ"
    NQ = "NQ"
    BJ = "BJ"


class SellReferType(IntEnum):
    """
    参考卖出类型
    highest-最高卖出价
    latest-最近卖出价
    """
    HIGHEST = 0  # 最高卖出价
    LATEST = 1  # 最近卖出价


class InvestmentSource(IntEnum):
    """
    投资资金来源
    """
    CASH = 0  # 现金
    FINANCING = 1  # 场内融资
    LOANED = 2  # 场外借贷


class SecuritySource(IntEnum):
    """
    证券获得方式
    """
    BUY = 0  # 正常买入
    BONUS_SHARES = 1  # 送股
    CONVERT_SHARES = 2  # 转增股


class StatusSell(IntEnum):
    """
    售出状态
    """

    UNSOLD = 0  # 未售出
    PARTIAL_SOLD = 1  # 部分售出
    ALL_SOLD = 2  # 全部售出


class BuyAndSellType(IntEnum):
    """
    卖出卖出记录类型
    """
    COMMISSION = 0  # 委托单
    CONDITION = 1  # 条件单
    CONTRACT = 2  # 成交单


class SecurityType(IntEnum):
    """
    证券类型
    """

    security_stock_a = 0  # A 股股票

    security_stock_b = 1  # B 股股票

    security_etf = 2  # 场内基金-ETF

    security_lof = 3  # 场内基金-LOF


class PushConfigModel(BaseModel):
    """
    驼峰转下线
    """
    device_id: int = None
    device_name: str = None
    eable_flag: str = None
    update_at: Optional[datetime] = None


class CashFlowType(IntEnum):
    INTO = 0  # 转入
    OUT = 1  # 转出


class AdminStatus(str, Enum):
    """
    管理员账户状态
    """

    DELETE = "delete"  # 删除
    NORMAL = "normal"  # 正常
    FORBIDDEN = "forbidden"  # 禁用
    UNAUTHORIZED = "unauthorized"  # 未验证


class CustomerSecurityAccountSecurityAdminStatus(str, Enum):
    """
    客户的证券账户中的券商状态
    """
    JOIN_GROUP = "join_group"  # 新入群
    ACCOUNT_READY_NOT_DEPOSIT = "account_ready_not_deposit"  # 已开户但未入金
    ACCOUNT_SUBSTANDARD = "account_substandard"  # 非有效户(20个交易日里平均市值1万)
    ACCOUNT_UNSETTLED = "account_unsettled"  # 待结算
    ACCOUNT_SETTLED = "account_settled"  # 已结算


class CustomerSecurityAccountVipStatus(str, Enum):
    """
    客户的证券账户中的vip状态
    """
    REWARDED = "rewarded"  # 已奖励
    UNREWARD = "unreward"  # 未奖励


class UserBenefitStatus(IntEnum):
    """
    用户福利领取状态
    """
    UNRECEIVED = 0  # 未领取
    RECEIVED = 1  # 已领取


class SecurityAdminTransferStatus(IntEnum):
    """
    券商转账的状态
    """
    PAYED = 1  # 已支付
    CONFIRMED = 2  # 已确认
    DISAGREED = 3  # 有异议


class YuanMaTalkQuestionPublicStatus(IntEnum):
    """
    问题是否公开
    """
    PUBLIC = 1  # 公开
    PRIVATE = 0  # 私密


class YuanMaTalkQuestionStatus(str, Enum):
    """
    问题状态
    """
    NEW = "new"  # 新问题
    REPLIED = "replied"  # 已回复
    WAITING = "waiting"  # 待回复
    CLOSED = "closed"  # 关闭
    DELETED = "deleted"  # 删除
    FORBIDDEN = "forbidden"  # 禁止


class LastReplySourceType(str, Enum):
    """
    问题状态
    """
    ADMIN = "admin"  # 管理后台
    USER = "user"  # 用户端


class AccountType(IntEnum):
    """
    账户类型
    """
    LIANGRONG = 1  # 两融账户
    ORDINARY = 0  # 普通用户


class supervisionFee(IntEnum):
    """
    深市是否申请监管费
    """
    NOT_CHARGE = 1  # 不收
    CHARGE = 0  # 收


class transferFee(IntEnum):
    """
    账户类型
    """
    NOT_CHARGE = 1  # 不收
    CHARGE = 0  # 收


class IsType(IntEnum):
    """
    是否默认
    """
    YES = 1  # 是
    NO = 0  # 否
