import typing
from typing import Generic, Sequence, TypeVar

from fastapi import status
from fastapi.encoders import jsonable_encoder
from fastapi.responses import JSONResponse
from pydantic import BaseModel

T = TypeVar("T")


class EntityResponse(BaseModel, Generic[T]):
    """实体数据"""
    status_code: int = 200
    message: str = "Success"
    data: T


class ListResponse(BaseModel, Generic[T]):
    """列表数据"""
    status_code: int = 200
    message: str = "Success"
    data: Sequence[T]


# 包装响应的 Pydantic 模型
# class EntityPageResponse(BaseModel, Generic[T]):
#     status_code: int
#     message: str
#     data: PageData


def response_entity_response(data, status_code=200, message="Success") -> EntityResponse:
    """普通实体类"""
    return EntityResponse(data=data, status_code=status_code, message=message)


# def response_page_response(data, status_code=200, message="Success") -> EntityPageResponse:
#     """普通分页类"""
#     return EntityPageResponse(data=data, status_code=status_code, message=message)


def response_list_response(data, status_code=200, message="Success") -> ListResponse:
    """普通列表数据"""
    return ListResponse(data=data, status_code=status_code, message=message)


def response_success(message: str = 'Success',
                     headers: typing.Optional[typing.Dict[str, str]] = None) -> JSONResponse:
    """成功返回"""
    return JSONResponse(
        status_code=status.HTTP_200_OK,
        headers=headers,
        content=jsonable_encoder({
            "status_code": status.HTTP_200_OK,
            "message": message,
        }))


def response_unauthorized() -> JSONResponse:
    """未登录"""
    return JSONResponse(
        status_code=status.HTTP_401_UNAUTHORIZED,
        headers={"WWW-Authenticate": "Bearer"},
        content=jsonable_encoder({
            "status_code": status.HTTP_401_UNAUTHORIZED,
            "message": '用户认证失败',
        }))
