import json  # 导入 json 库

from fastapi import APIRouter, HTTPException  # 从 FastAPI 中导入 APIRouter，用于创建 API 路由器

import src.xtdata.service as service  # 导入服务模块，该模块包含各种服务的实现
from src.pydantic.codelistrequest import CodeListRequest  # 导入 Pydantic 模型，用于验证请求参数
from src.pydantic.factor_request import StockQuery
from src.pydantic.request_data import DataRequest  # 导入 Pydantic 模型，用于验证请求参数
from src.responses import response_entity_response, response_list_response  # 导入自定义响应格式化函数

router = APIRouter()  # 创建一个 FastAPI 路由器实例


# 获取完整的逐笔成交键
@router.get("/get_full_tick_keys")
async def get_full_tick_keys(request: CodeListRequest):
    """
    获取完整的逐笔成交键。
    """
    result = await service.get_full_tick_keys_service(request.code_list)  # 调用服务获取逐笔成交键
    return result  # 返回获取的结果


# 获取完整的逐笔成交数据
@router.get("/get_full_tick")
async def get_full_tick(request: CodeListRequest):
    """
    获取完整的逐笔成交数据。
    """
    result = await service.get_full_tick_service(request.code_list)  # 调用服务获取逐笔成交数据
    return result  # 返回获取的结果


# 获取市场数据
@router.get("/get_market_data")
async def get_market_data(request: DataRequest):
    """
    获取市场数据。
    """
    # 调用服务获取市场数据
    market_data = await service.get_market_data_service(field_list=request.field_list,
                                                        stock_list=request.stock_list,
                                                        period=request.period,
                                                        start_time=request.start_time,
                                                        end_time=request.end_time,
                                                        count=request.count,
                                                        dividend_type=request.dividend_type,
                                                        fill_data=request.fill_data)
    return market_data  # 返回市场数据


# 获取完整的 K 线数据
@router.get("/get_full_kline")
async def get_full_kline(request: DataRequest):
    """
    获取完整的 K 线数据。
    """
    # 调用服务获取完整的 K 线数据
    result = await service.get_full_kline_service(field_list=request.field_list,
                                                  stock_list=request.stock_list,
                                                  period=request.period,
                                                  start_time=request.start_time,
                                                  end_time=request.end_time,
                                                  count=request.count,
                                                  dividend_type=request.dividend_type,
                                                  fill_data=request.fill_data)
    return result  # 返回 K 线数据


# 获取合约详情
@router.get("/get_instrument_detail")
async def get_instrument_detail(request: DataRequest):
    """
    获取合约的详细信息。
    """
    # 调用服务获取合约详情数据
    result = await service.get_instrument_detail_service(stock_code=request.stock_code,
                                                         iscomplete=request.iscomplete)
    return result  # 返回合约详情


@router.get("/get_stock_factor")
async def get_stock_factor(query_params: StockQuery):
    stocks = await service.get_stock_factor_service(query_params)
    return stocks


# 获取本地数据
@router.get("/get_local_data")
async def get_local_data(request: DataRequest):
    """
    获取本地存储的股票数据。
    """
    # 调用服务获取本地数据
    result = await service.get_local_data_service(field_list=request.field_list, stock_list=request.stock_list,
                                                  period=request.period, start_time=request.start_time,
                                                  end_time=request.end_time, count=request.count,
                                                  dividend_type=request.dividend_type, fill_data=request.fill_data,
                                                  data_dir=request.data_dir)
    return result


# 获取板块列表
@router.post("/get_sector_list")
async def get_sector_list():
    """
    获取所有板块的列表。
    """
    # 调用服务获取板块列表
    result = await service.get_sector_list_service()
    return result  # 返回板块列表


# 获取某板块中的股票列表
@router.get("/get_stock_list_in_sector")
async def get_stock_list_in_sector(request: DataRequest):
    """
    获取指定板块中的股票列表。
    """
    # 调用服务获取板块中的股票列表
    result = await service.get_stock_list_in_sector_service(sector_name=request.sector_name)
    return result  # 返回股票列表


# 下载板块数据
@router.post("/download_sector_data")
async def download_sector_data():
    """
    下载板块的详细数据。
    """
    # 调用服务下载板块数据
    result = await service.download_sector_data_service()
    return result  # 返回下载结果


# 订阅股票报价
@router.post("/subscribe_quote")
async def subscribe_quote(request: DataRequest):
    """
    订阅单只股票的报价。
    """
    # 调用服务订阅股票报价
    result = await service.subscribe_quote_service(stock_code=request.stock_code,
                                                   period=request.period,
                                                   start_time=request.start_time,
                                                   end_time=request.end_time,
                                                   count=request.count,
                                                   callback=request.callback)
    return response_list_response(data=result, status_code=200, message="Success")  # 返回响应


# 订阅所有股票报价
@router.post("/subscribe_whole_quote")
async def subscribe_whole_quote(request: DataRequest):
    """
    订阅所有股票的报价。
    """
    # 调用服务订阅所有股票的报价
    result = await service.subscribe_whole_quote_service(code_list=request.code_list,
                                                         callback=request.callback)
    # return result  # 返回响应
    return response_list_response(data=result, status_code=200, message="Success")  # 返回响应


# 下载历史数据
@router.post("/download_history_data")
async def download_history_data(request: DataRequest):
    """
    下载指定股票的历史数据。
    """
    # 调用服务下载指定股票的历史数据
    await service.download_history_data_service(stock_code=request.stock_code,
                                                period=request.period,
                                                start_time=request.start_time,
                                                end_time=request.end_time,
                                                incrementally=request.incrementally)
    return response_list_response(data=[], status_code=200, message="Success")  # 返回响应


# 批量下载历史数据
@router.post("/batch_download_history_data")
async def download_history_data(request: DataRequest):
    """
    批量下载多个股票的历史数据。
    """
    # 调用服务批量下载历史数据
    await service.download_history_data2_service(stock_list=request.stock_list,
                                                 period=request.period,
                                                 start_time=request.start_time,
                                                 end_time=request.end_time,
                                                 callback=request.callback)
    return response_list_response(data=[], status_code=200, message="Success")  # 返回响应


# 批量下载历史数据
@router.post("/update_stock_data")
async def update_stock_data():
    """
    批量下载多个股票的历史数据。
    """
    # 调用服务批量下载历史数据
    await service.update_stock_data_service()
    return response_list_response(data=[], status_code=200, message="Success")  # 返回响应
