import numpy as np  # 导入 numpy 库
from xtquant import xtdata  # 导入 xtquant 库的 xtdata 模块

from src.utils.backtest_until import convert_pandas_to_json_serializable
from src.models.wance_data_stock import WanceDataStock
from src.pydantic.factor_request import StockQuery
from src.utils.history_data_processing_utils import translation_dict


# 获取股票池中的所有股票的股票代码
async def get_full_tick_keys_service(code_list: list):
    """
    获取所有股票的逐笔成交键。若未提供股票列表，则默认使用 ['SH', 'SZ']。
    """
    if len(code_list) == 0:  # 如果股票代码列表为空
        code_list = ['SH', 'SZ']  # 默认使用上证和深证股票池
    result = xtdata.get_full_tick(code_list=['SH', 'SZ'])  # 调用 xtdata 库的函数获取逐笔成交数据
    return list(result.keys())  # 返回所有股票代码的列表


# 获取股票池中的所有股票的逐笔成交数据
async def get_full_tick_service(code_list: list):
    """
    获取所有股票的逐笔成交数据。若未提供股票列表，则默认使用 ['SH', 'SZ']。
    """
    if len(code_list) == 0:  # 如果股票代码列表为空
        code_list = ['SH', 'SZ']  # 默认使用上证和深证股票池
    result = xtdata.get_full_tick(code_list=code_list)  # 获取逐笔成交数据
    return result  # 返回逐笔成交数据


# 获取板块信息
async def get_sector_list_service():
    """
    获取所有板块的列表。
    """
    result = xtdata.get_sector_list()  # 调用 xtdata 库的函数获取板块信息
    return result  # 返回板块列表


# 通过板块名称获取股票列表
async def get_stock_list_in_sector_service(sector_name):
    """
    获取指定板块中的所有股票名称。
    """
    result = xtdata.get_stock_list_in_sector(sector_name=sector_name)  # 根据板块名称获取股票列表
    return result  # 返回股票列表


# 下载板块信息
async def download_sector_data_service():
    """
    下载板块的详细数据。
    """
    result = xtdata.download_sector_data()  # 调用 xtdata 库的函数下载板块数据
    return result  # 返回下载结果


# 获取股票的详细信息
async def get_instrument_detail_service(stock_code: str, iscomplete: bool):
    """
    获取股票的详细信息。
    """
    result = xtdata.get_instrument_detail(stock_code=stock_code, iscomplete=iscomplete)  # 获取股票的详细信息
    return result  # 返回股票的详细信息


# 获取市场行情数据
async def get_market_data_service(field_list: list, stock_list: list, period: str, start_time: str, end_time: str,
                                  count: int, dividend_type: str, fill_data: bool):
    """
    获取指定条件下的市场行情数据。
    """
    result = xtdata.get_market_data(field_list=field_list, stock_list=stock_list, period=period, start_time=start_time,
                                    end_time=end_time, count=count, dividend_type=dividend_type,
                                    fill_data=fill_data)  # 获取市场数据
    return result  # 返回市场数据


# 获取最新的 K 线数据
async def get_full_kline_service(field_list: list, stock_list: list, period: str, start_time: str, end_time: str,
                                 count: int,
                                 dividend_type: str, fill_data: bool):
    """
    获取指定条件下的完整 K 线数据。
    """
    result = xtdata.get_full_kline(field_list=field_list, stock_list=stock_list, period=period, start_time=start_time,
                                   end_time=end_time, count=count, dividend_type=dividend_type,
                                   fill_data=fill_data)  # 获取完整的 K 线数据
    return result  # 返回 K 线数据


# 下载历史数据
async def download_history_data_service(stock_code: str, period: str, start_time: str, end_time: str,
                                        incrementally: bool):
    """
    下载指定股票的历史数据。
    """
    xtdata.download_history_data(stock_code=stock_code, period=period, start_time=start_time, end_time=end_time,
                                 incrementally=incrementally)  # 下载股票历史数据


# 获取本地数据
async def get_local_data_service(field_list: list, stock_list: list, period: str, start_time: str, end_time: str,
                                 count: int, dividend_type: str, fill_data: bool, data_dir: str):
    """
    @param field_list:
    @type field_list:
    @param stock_list:
    @type stock_list:
    @param period:
    @type period:
    @param start_time:
    @type start_time:
    @param end_time:
    @type end_time:
    @param count:
    @type count:
    @param dividend_type:
    @type dividend_type:
    @param fill_data:
    @type fill_data:
    @param data_dir:
    @type data_dir:
    @return:
    @rtype:
    """
    """
    获取本地存储的股票数据。
    """
    return_list = []
    result = xtdata.get_local_data(field_list=field_list, stock_list=stock_list, period=period, start_time=start_time,
                          end_time=end_time, count=count, dividend_type=dividend_type, fill_data=fill_data,
                          data_dir=data_dir)  # 获取本地数据
    for i in stock_list:
        return_list.append({
            i : convert_pandas_to_json_serializable(result.get(i))
        })
    return return_list


# 订阅全市场行情推送
async def subscribe_whole_quote_service(code_list, callback=None):
    """
    订阅全市场行情推送。
    """
    if callback:
        result = xtdata.subscribe_whole_quote(code_list, callback=on_data)  # 有回调函数时调用
    else:
        result = xtdata.subscribe_whole_quote(code_list, callback=None)  # 无回调函数时调用
    return result  # 返回订阅结果


# 订阅单只股票行情推送
async def subscribe_quote_service(stock_code: str, period: str, start_time: str, end_time: str, count: int,
                                  callback: bool):
    """
    订阅单只股票的行情推送。
    """
    if callback:
        result = xtdata.subscribe_quote(stock_code=stock_code, period=period, start_time=start_time, end_time=end_time,
                                        count=count, callback=on_data)  # 有回调函数时调用
    else:
        result = xtdata.subscribe_quote(stock_code=stock_code, period=period, start_time=start_time, end_time=end_time,
                                        count=count, callback=None)  # 无回调函数时调用
    return result  # 返回订阅结果


def on_data(datas):
    """
    行情数据推送的回调函数，处理并打印接收到的行情数据。
    """
    for stock_code in datas:
        print(stock_code, datas[stock_code])  # 打印接收到的行情数据


# 批量增量下载或批量下载
async def download_history_data2_service(stock_list: list, period: str, start_time: str, end_time: str, callback: bool):
    """
    批量下载股票历史数据，支持进度回调。
    """
    if callback:
        xtdata.download_history_data2(stock_list=stock_list, period=period, start_time=start_time, end_time=end_time,
                                      callback=on_progress)  # 有回调函数时调用
    else:
        xtdata.download_history_data2(stock_list=stock_list, period=period, start_time=start_time, end_time=end_time,
                                      callback=None)  # 无回调函数时调用


def on_progress(data):
    """
    数据下载进度的回调函数，实时显示进度。
    """
    print(data)  # 打印下载进度


# 单只股票下载示例
def download_data():
    """
    测试函数：下载单只股票的历史数据并获取相关信息。
    """
    xtdata.download_history_data("000300.SH", "1d", start_time='', end_time='', incrementally=None)  # 下载历史数据
    a = xtdata.get_full_tick(code_list=['SH', 'SZ'])  # 获取逐笔成交数据
    print(list(a.keys()))  # 打印获取的股票代码


async def update_stock_data_service():
    """
    # 用于本地xtdata数据更新
    @return:
    @rtype:
    """
    stock_list = await get_full_tick_keys_service(['SH', 'SZ'])
    for stock in stock_list:
        await download_history_data_service(stock_code=stock,
                                             period='1d',
                                             start_time='',
                                             end_time='',
                                             incrementally=True)


async def get_stock_factor_service(query_params: StockQuery):
    """
    动态查询方法，根据传入的字段和值进行条件构造
    :param kwargs: 查询条件，键为字段名，值为查询值
    :return: 符合条件的记录列表
    """
    """
    使用示例：
    {
      "financial_asset_value": 10.7169,
      "financial_cash_flow": -0.42982,
      "gt": {
        "profit_gross_rate": 4.0,
        "valuation_PB_TTM": 0.8
      },
      "lt": {
        "market_indicator": -0.6
      },
      "gte": {
        "valuation_market_percentile":5.38793
      },
      "lte": {
        "valuation_PEGTTM_ratio": 15.2773
      },
      "between": {
        "financial_dividend": {
          "min": 1.0,
          "max": 2.0
        },
        "growth_Income_rate": {
          "min": -0.6,
          "max": 1.0
        }
      },
      "stock_code": "600051.SH",
      "stock_name": "宁波联合",
      "stock_sector": ["上证A股", "沪深A股"],
      "stock_type": ["stock"],
      "time_start": "0",
      "time_end": "20240911",
      "valuation_PEG_percentile": 15.2222,
      "valuation_PTS_TTM": 2.3077
    }

    
    financial_asset_value 和 financial_cash_flow 字段为普通等于条件。
    gt 字段用于指定字段大于某个值的条件，例如 profit_gross_rate 大于 15.0。
    lt 字段用于指定字段小于某个值的条件，例如 market_indicator 小于 30.0。
    gte 字段用于指定字段大于等于某个值的条件，例如 valuation_market_percentile 大于等于 0.5。
    lte 字段用于指定字段小于等于某个值的条件，例如 valuation_PEGTTM_ratio 小于等于 2.0。
    between 字段用于指定字段在两个值之间的范围，例如 financial_dividend 在 1.0 和 5.0 之间，growth_Income_rate 在 10.0 和 25.0 之间。
    stock_code 和 stock_name 字段为普通等于条件。
    stock_sector 和 stock_type 字段为列表类型的条件，可能会进行包含匹配。
    time_start 和 time_end 字段为普通等于条件。
    valuation_PEG_percentile 和 valuation_PTS_TTM 字段为普通等于条件。

    比较大小字段说明：
        'gt' 对应 __gt（大于）
        'lt' 对应 __lt（小于）
        'gte' 对应 __gte（大于等于）
        'lte' 对应 __lte（小于等于）
        'between' 对应在某个值之间的条件，使用 __gt 和 __lt 组合实现
        
    时间字段说明：
        time_start={'recent_years': 2} 近2年
        time_start={'recent_months': 2} 近2月
    """

    filters = {}

    # 处理大于、小于等操作
    if query_params.gt:
        for field, value in query_params.gt.items():
            filters[f"{field}__gt"] = value
    if query_params.lt:
        for field, value in query_params.lt.items():
            filters[f"{field}__lt"] = value
    if query_params.gte:
        for field, value in query_params.gte.items():
            filters[f"{field}__gte"] = value
    if query_params.lte:
        for field, value in query_params.lte.items():
            filters[f"{field}__lte"] = value

    # 处理范围查询
    if query_params.between:
        for field, range_values in query_params.between.items():
            if 'min' in range_values and 'max' in range_values:
                filters[f"{field}__gte"] = range_values['min']
                filters[f"{field}__lte"] = range_values['max']

    # 处理普通字段
    for field, value in query_params.model_dump(exclude_unset=True).items():
        if field not in ['gt', 'lt', 'gte', 'lte', 'between']:
            if isinstance(value, float):
                filters[f"{field}__gte"] = value
            elif isinstance(value, str):
                filters[f"{field}__icontains"] = value
            elif isinstance(value, list):
                if field == "stock_sector":
                    # 转换 `stock_sector` 中的中文板块名到英文
                    translated_sectors = [translation_dict.get(sector, sector) for sector in value]
                    filters[f"{field}__contains"] = translated_sectors
                else:
                    filters[f"{field}__contains"] = value

    try:
        stocks = await WanceDataStock.filter(**filters).all()
        return stocks
    except Exception as e:
        print(f"Error occurred when querying stocks: {e}")
        return []


# 将 numpy 类型转换为原生 Python 类型
async def convert_numpy_to_native(obj):
    """
    递归地将 numpy 类型转换为原生 Python 类型。
    """
    if isinstance(obj, np.generic):  # 如果是 numpy 类型
        return obj.item()  # 转换为 Python 原生类型
    elif isinstance(obj, dict):  # 如果是字典类型
        return {k: convert_numpy_to_native(v) for k, v in obj.items()}  # 递归转换字典中的值
    elif isinstance(obj, list):  # 如果是列表类型
        return [convert_numpy_to_native(i) for i in obj]  # 递归转换列表中的值
    elif isinstance(obj, tuple):  # 如果是元组类型
        return tuple(convert_numpy_to_native(i) for i in obj)  # 递归转换元组中的值
    else:
        return obj  # 如果是其他类型，直接返回


if __name__ == '__main__':
    # result = asyncio.run(get_instrument_detail_service("000300.SH",False))
    # print(result)
    # resulta = asyncio.run(get_sector_list_service())
    # print(resulta)
    pass
